{ *** A simple Epson POS Clock.
  *** Sends output via I2C UART.
  *** Martin Hepperle, September 2018
  *** }
Program POSClock;

Const
  I2C_UART_BASE = $50; { base port address of I2C register set }
  I2C_UART_THR  = $0;  { transmit hold register }
  I2C_UART_LCR  = $3;  { line control register }
  I2C_UART_LSR  = $5;  { line status register }

Type String255 = String[255];

Var
  Running:  Boolean;
  hh,mm,ss: Byte;
  ssPrev:   Byte;
  Buffer:   String[8];

{ ------------------------------------- }

Function GetBaudRate: Real;
  { --- return the I2C-UART baud rate }
Var
  R:      Byte;

Begin

  { -- save current settings }
  R := Port[I2C_UART_BASE + I2C_UART_LCR];

  { --- allow access to special registers 0 and 1 }
  Port[I2C_UART_BASE + I2C_UART_LCR] := 128;

  { --- set divisor for baud rate (for 14.7456 MHz crystal) }
  GetBaudRate := 921600.0 / ( Port[I2C_UART_BASE + $00] * 1.0 +
                              Port[I2C_UART_BASE + $01] * 256 );

  { --- disallow access special registers 0 and 1 }
  Port[I2C_UART_BASE + I2C_UART_LCR] := R;

End;

{ ------------------------------------- }

Procedure SendByte ( Data: Byte );
{ Wait for UART ready, then place a byte into its FIFO buffer }
Var
  Status: Byte;
Begin
    Repeat
      Status := Port[I2C_UART_BASE + I2C_UART_LSR];
    Until Status and 32 = 32; { THR empty? }
    Port[I2C_UART_BASE + I2C_UART_THR] := Data;
End;

{ ------------------------------------- }

Procedure SendString ( Buffer: String255 );
{ Send a whole string of bytes to the UART }
Var
  i:      Integer;
Begin
    { --- output this line }
    For i:=1 To Length(Buffer) Do
    Begin
      SendByte( Ord(Buffer[i]) );
    End;
End;

{ ------------------------------------- }

Begin

  If GetBaudRate < 100 Then
  Begin
    WriteLn('ERROR: Baud rate not set properly - use MODE first.');
    Exit;
  End;

  { cursor OFF }
  SendString(Chr(31)+'C'+Chr(0));
  { clear display and title line }
  SendString(Chr(12)+'AVR-CP/M ----- Clock');

  ssPrev := 255;

  Buffer[0] := Chr(8); { string length }
  Buffer[3] := ':';
  Buffer[6] := ':';

  Running := True;

  While Running Do
  Begin

    ss := Port[$47];

    If ss <> ssPrev Then
    Begin
      { display new time }
      mm := Port[$48];
      hh := Port[$49];

      ssPrev := ss;

      { BCD to ASCII }
      Buffer[8] := Chr(48 + (ss And $0F));
      Buffer[7] := Chr(48 + (ss shr 4));
      Buffer[5] := Chr(48 + (mm And $0F));
      Buffer[4] := Chr(48 + (mm shr 4));
      Buffer[2] := Chr(48 + (hh And $0F));
      Buffer[1] := Chr(48 + (hh shr 4));

      { position cursor }
      SendString(Chr(31)+'$'+Chr(7)+Chr(2));

      { display time 00:00:00 }
      SendString(Buffer);

      { check for key press }
      If Bdos($0B) <> 0 Then
      Begin
        { swallow character }
        Bdos ($01);
        Running := False;
      End;

    End;

    Delay(100);

  End;

End.
